#!/bin/bash
set -e

DOCKER_DIR="$(dirname $(realpath $0))"
. ${DOCKER_DIR}/env.sh

cd $DS_OS_DIR

MODE=none
#MODE=asan
#MODE=tsan

do_make() {
    pushd $1
    make PREFIX=$PREFIX install
    popd
}

do_autoconf() {
    pushd $1
    autoreconf -i
    mkdir -p build-${MODE}
    cd build-${MODE}
    case $MODE in
      none) ../configure --prefix=${PREFIX};;
      asan) ../configure --prefix=${PREFIX} --enable-asan;;
      tsan) ../configure --prefix=${PREFIX} --enable-tsan;;
    esac
    make -j $(nproc)
    make install
    popd
}

do_autoconf_broken_sep() {
    pushd $1
    autoreconf -i
    case $MODE in
      none) ./configure --prefix=${PREFIX};;
      asan) ./configure --prefix=${PREFIX} --enable-asan;;
      tsan) ./configure --prefix=${PREFIX} --enable-tsan;;
    esac
    make -j $(nproc)
    make install
    popd
}

do_distclean() {
    pushd $1
    make distclean
    popd
}

do_cmake() {
    # aptitude install swig python-dev
    pushd $1
    mkdir -p build-${MODE}
    # missing lua library
    case $MODE in
      none) cmake -Bbuild-${MODE} -DCMAKE_INSTALL_PREFIX=$PREFIX -DWITH_SCRIPTING=off;;
      asan) cmake -Bbuild-${MODE} -DCMAKE_INSTALL_PREFIX=$PREFIX -DWITH_SCRIPTING=off -DSANITIZE_ADDRESS=On;;
      tsan) cmake -Bbuild-${MODE} -DCMAKE_INSTALL_PREFIX=$PREFIX -DWITH_SCRIPTING=off -DSANITIZE_THREAD=On;;
    esac
    make -C build-${MODE} -j $(nproc)
    make -C build-${MODE} install
    popd
}

for i in \
    libdsuid \
    libds485-core \
    libds485-netlib \
    libds485-client \
; do
    do_autoconf $i
done

do_cmake libdsm-api


# not ds485 stack related
do_autoconf libcommchannel
do_autoconf_broken_sep dss-websrc-mainline
do_make dsdata
