/*
    Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "defaultbuseventsink.h"

#include "model/apartment.h"
#include "model/modelmaintenance.h"

namespace dss {

DefaultBusEventSink::DefaultBusEventSink(std::shared_ptr<Apartment> _pApartment,
                                         std::shared_ptr<ModelMaintenance> _pModelMaintenance)
    : m_pApartment(_pApartment), m_pModelMaintenance(_pModelMaintenance) {}

void DefaultBusEventSink::onGroupCallScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID,
                                           const int _groupID, const int _originDeviceId, const int _sceneID,
                                           Caller&& caller, const bool _force) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etCallSceneGroup, _dsMeterID);
  pEvent->addParameter(_zoneID);
  pEvent->addParameter(_groupID);
  pEvent->addParameter(_originDeviceId);
  pEvent->addParameter(_sceneID);
  pEvent->addParameter(caller.origin);
  pEvent->addParameter(_force);
  pEvent->setSingleStringParameter(caller.token);
  pEvent->addParameter(false); /* not a "real" min scene call */
  m_pModelMaintenance->addModelEvent(pEvent);
} // onGroupCallScene

void DefaultBusEventSink::onGroupCallSceneMin(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID,
                                              const int _groupID, const int _originDeviceId, const int _sceneID,
                                              Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etCallSceneGroup, _dsMeterID);
  pEvent->addParameter(_zoneID);
  pEvent->addParameter(_groupID);
  pEvent->addParameter(_originDeviceId);
  pEvent->addParameter(_sceneID);
  pEvent->addParameter(caller.origin);
  pEvent->addParameter(false); /* force not applicable */
  pEvent->setSingleStringParameter(caller.token);
  pEvent->addParameter(true); /* "real" min scene call */
  m_pModelMaintenance->addModelEvent(pEvent);
} // onGroupCallScene

void DefaultBusEventSink::onGroupSaveScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID,
                                           const int _groupID, const int _originDeviceId, const int _sceneID,
                                           Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etZoneGroupSaveScene, _dsMeterID);
  pEvent->addParameter(_zoneID);
  pEvent->addParameter(_groupID);
  pEvent->addParameter(_originDeviceId);
  pEvent->addParameter(_sceneID);
  pEvent->addParameter(caller.origin);
  pEvent->setSingleStringParameter(caller.token);
  m_pModelMaintenance->addModelEvent(pEvent);
}

void DefaultBusEventSink::onGroupUndoScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID,
                                           const int _groupID, const int _originDeviceId, const int _sceneID,
                                           const bool _explicit, Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etUndoSceneGroup, _dsMeterID);
  pEvent->addParameter(_zoneID);
  pEvent->addParameter(_groupID);
  pEvent->addParameter(_originDeviceId);
  if (_explicit) {
    pEvent->addParameter(_sceneID);
  } else {
    pEvent->addParameter(-1);
  }
  pEvent->addParameter(caller.origin);
  pEvent->setSingleStringParameter(caller.token);
  m_pModelMaintenance->addModelEvent(pEvent);
} // onGroupUndoScene

void DefaultBusEventSink::onGroupBlink(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID,
                                       const int _groupID, const int _originDeviceId, Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etBlinkGroup, _dsMeterID);
  pEvent->addParameter(_zoneID);
  pEvent->addParameter(_groupID);
  pEvent->addParameter(_originDeviceId);
  pEvent->addParameter(caller.origin);
  pEvent->setSingleStringParameter(caller.token);
  m_pModelMaintenance->addModelEvent(pEvent);
} // onGroupCallScene

void DefaultBusEventSink::onDeviceCallScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID,
                                            const int _originDeviceId, const int _sceneID, Caller&& caller,
                                            const bool _force) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etCallSceneDevice, _dsMeterID);
  pEvent->addParameter(_deviceID);
  pEvent->addParameter(_originDeviceId);
  pEvent->addParameter(_sceneID);
  pEvent->addParameter(caller.origin);
  pEvent->addParameter(_force);
  pEvent->setSingleStringParameter(caller.token);
  m_pModelMaintenance->addModelEvent(pEvent);
} // onDeviceCallScene

void DefaultBusEventSink::onDeviceSaveScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID,
                                            const int _originDeviceId, const int _sceneID, Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etSaveSceneDevice, _dsMeterID);
  pEvent->addParameter(_deviceID);
  pEvent->addParameter(_originDeviceId);
  pEvent->addParameter(_sceneID);
  pEvent->addParameter(caller.origin);
  pEvent->setSingleStringParameter(caller.token);
  m_pModelMaintenance->addModelEvent(pEvent);
} // onDeviceCallScene

void DefaultBusEventSink::onDeviceUndoScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID,
                                            const int _originDeviceId, const int _sceneID, const bool _explicit,
                                            Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etUndoSceneDevice, _dsMeterID);
  pEvent->addParameter(_deviceID);
  pEvent->addParameter(_originDeviceId);
  if (_explicit) {
    pEvent->addParameter(_sceneID);
  } else {
    pEvent->addParameter(-1);
  }
  pEvent->addParameter(caller.origin);
  pEvent->setSingleStringParameter(caller.token);
  m_pModelMaintenance->addModelEvent(pEvent);
} // onDeviceUndoScene

void DefaultBusEventSink::onDeviceBlink(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID,
                                        const int _originDeviceId, Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etBlinkDevice, _dsMeterID);
  pEvent->addParameter(_deviceID);
  pEvent->addParameter(_originDeviceId);
  pEvent->addParameter(caller.origin);
  pEvent->setSingleStringParameter(caller.token);
  m_pModelMaintenance->addModelEvent(pEvent);
} // onDeviceCallScene

void DefaultBusEventSink::onMeteringEvent(BusInterface* _source, const dsuid_t& _dsMeterID, const unsigned int _powerW,
                                          const unsigned int _energyWs, const unsigned int _powerProducedW,
                                          const unsigned int _energyProducedWs) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etMeteringValues, _dsMeterID);
  pEvent->addParameter(_powerW);
  pEvent->addParameter(_energyWs);
  pEvent->addParameter(_powerProducedW);
  pEvent->addParameter(_energyProducedWs);
  m_pModelMaintenance->addModelEvent(pEvent);
}

void DefaultBusEventSink::onZoneSensorValue(BusInterface* _source, const dsuid_t& _dsMeterID,
                                            const dsuid_t& _sourceDevice, const int& _zoneID, const int& _groupID,
                                            SensorType _sensorType, const int& _sensorValue, const int& _precision,
                                            Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithStrings(ModelEvent::etZoneSensorValue, _dsMeterID);
  pEvent->addParameter(_zoneID);
  pEvent->addParameter(_groupID);
  pEvent->addParameter(static_cast<int>(_sensorType));
  pEvent->addParameter(_sensorValue);
  pEvent->addParameter(_precision);
  pEvent->setSingleStringParameter(dsuid2str(_sourceDevice));
  m_pModelMaintenance->addModelEvent(pEvent);
} // onZoneSensorValue

void DefaultBusEventSink::onDeviceSetValue(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID,
                                           const uint8_t _value, Caller&& caller) {
  ModelEvent* pEvent = new ModelEventWithDSID(ModelEvent::etDeviceSetOutputValue, _dsMeterID);
  pEvent->addParameter(_deviceID);
  pEvent->addParameter(_value);
  pEvent->addParameter(caller.origin);
  m_pModelMaintenance->addModelEvent(pEvent);
}

void DefaultBusEventSink::onZoneGroupSetValue(BusInterface* _source, const int _zoneID, const int _groupID,
                                              const uint8_t _value, Caller&& caller) {
  ModelEvent* pEvent = new ModelEvent(ModelEvent::etZoneGroupSetOutputValue);
  pEvent->addParameter(_zoneID);
  pEvent->addParameter(_groupID);
  pEvent->addParameter(_value);
  pEvent->addParameter(caller.origin);
  m_pModelMaintenance->addModelEvent(pEvent);
}

} // namespace dss
