/*
    Copyright (c) 2010,2014 digitalSTROM.org, Zurich, Switzerland

    Authors: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>,
             Sergey Bostandzhyna <jin@dev.digitalstrom.org>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include "businterface.h"

namespace dss {

class ModelMaintenance;
class Apartment;

class DefaultBusEventSink : public BusEventSink {
public:
  DefaultBusEventSink(std::shared_ptr<Apartment> _pApartment, std::shared_ptr<ModelMaintenance> _pModelMaintenance);
  virtual ~DefaultBusEventSink(){};

  void onGroupCallScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID, const int _groupID,
                        const int _originDeviceId, const int _sceneID, Caller&& caller, const bool _force) override;
  void onGroupCallSceneMin(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID, const int _groupID,
                           const int _originDeviceId, const int _sceneID, Caller&& caller) override;
  void onGroupSaveScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID, const int _groupID,
                        const int _originDeviceId, const int _sceneID, Caller&& caller) override;
  void onGroupUndoScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID, const int _groupID,
                        const int _originDeviceId, const int _sceneID, const bool _explicit, Caller&& caller) override;
  void onGroupBlink(BusInterface* _source, const dsuid_t& _dsMeterID, const int _zoneID, const int _groupID,
                    const int _originDeviceId, Caller&& caller) override;
  void onDeviceCallScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID,
                         const int _originDeviceId, const int _sceneID, Caller&& caller, const bool _force) override;
  void onDeviceSaveScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID,
                         const int _originDeviceId, const int _sceneID, Caller&& caller) override;
  void onDeviceBlink(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID, const int _originDeviceId,
                     Caller&& caller) override;
  void onDeviceUndoScene(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID,
                         const int _originDeviceId, const int _sceneID, const bool _explicit, Caller&& caller) override;
  void onMeteringEvent(BusInterface* _source, const dsuid_t& _dsMeterID, const unsigned int _powerW,
                       const unsigned int _energyWs, const unsigned int _powerProducedW,
                       const unsigned int _energyProducedWs) override;
  void onZoneSensorValue(BusInterface* _source, const dsuid_t& _dsMeterID, const dsuid_t& _sourceDevice,
                         const int& _zoneID, const int& _groupID, SensorType _sensorType, const int& _sensorValue,
                         const int& _precision, Caller&& caller) override;
  void onDeviceSetValue(BusInterface* _source, const dsuid_t& _dsMeterID, const int _deviceID, const uint8_t _value,
                        Caller&& caller) override;
  void onZoneGroupSetValue(BusInterface* _source, const int _zoneID, const int _groupID, const uint8_t _value,
                           Caller&& caller) override;

private:
  std::shared_ptr<Apartment> m_pApartment;
  std::shared_ptr<ModelMaintenance> m_pModelMaintenance;
};

} // namespace dss
