// Generated file.  DO NOT EDIT!
// source: meterings.yaml

// clang-format off

#pragma once


/// Meterings
///
/// Metering equipment is integrated into the digitalSTROM model in the installation group. Meters typically require special-purpose hardware with interface to digitalSTROM, but additionally all digitalSTROM Meters (DSM) support measuring of mains power consumption.
///
/// Three kind of metering data is available:
/// - measurements with highest resolution but short time,
/// - current values of all meters,
/// - aggregated data with long term history.
///
/// The highest resolution values are available on the DSS only. The period of time depends on the component. For DSM measurements the time period shall be at least 15 minutes with data point every second.
///
/// The long term history is kept in the HomeWiki. Typically for electrical meters there is one value stored every 15 minutes.
///
/// ## Examples of api request
///
/// This section is not an api specification. It is more a guide.
///
/// All examples are against apartment base url `/v1/apartments/{apartmentId}`.
///
/// ### Collection of all meterings
/// in the whole apartment / installation.
///
/// ```json
/// GET /installation/meterings
/// {
///   "data" : {
///     "meterings":[
///       {"id":"apartment-power","type":"powerMetering","attributes":{
///         "unit":"W","technicalName":"apartment power","origin":{"type":"apartment"}}},
///       {"id":"apartment-energy","type":"energyMetering","attributes":{
///         "unit":"Wh","technicalName":"apartment energy","origin":{"type":"apartment"}}},
///       {"id":"dsm-dsuid1-power","type":"powerMetering","attributes":{
///         "unit":"W","technicalName":"Dsm1 power","origin":{"type":"controller","id":"dsuid1"}}},
///       {"id":"dsm-dsuid1-energy","type":"energyMetering","attributes":{
///         "unit":"Wh","technicalName":"Dsm1 energy","origin":{"type":"controller","id":"dsuid1"}}},
///       {"id":"open-metering-system-energy-produced",type="energyMetering",
///         "attributes":{"unit":"Wh","origin:{"type":"open-metering-system","id":"serial-number"}}}
///       {"id":"open-metering-system-energy-produced-tariff1",type="electricity-energy-produced",
///         "attributes":{"unit":"Wh","origin:{"type":"open-metering-system","id":"serial-number"}}}
///     ]
///   }
/// }
/// GET /installation/meterings/apartment-power-produced
/// {
///   "data" : {
///     "id":"dsm-dsuid1-power","type":"powerMetering","attributes":{
///       "unit":"W","technicalName":"Dsm1 power","origin":{"type":"controller","id":"dsuid1"}
///     }
///   }
/// }
/// ```
/// GET /installation/meterings/apartment-power-produced
///
///
///
/// ### Collection of all metering values
/// in the whole apartment / installation.
///
/// Metering value id matches 1:1 to metering id.
///
/// ```json
/// GET /installation/meterings/values
/// {
///   "data": {
///     "values":[
///       {"id":"apartment-power","type":"meteringValue","lastChanged":"...", "attributes":{"value":11.2}},
///       {"id":"apartment-energy","type":"meteringValue","lastChanged":"...", "attributes":{"value":17.8}},
///       {"id":"dsm-dsuid1-power","type":"meteringValue","lastChanged":"...", "attributes":{"value":11.2}},
///       {"id":"dsm-dsuid1-energy","type":"meteringValue","lastChanged":"...", "attributes":{"value":17.8}}
///     ]
///   }
/// }
/// GET /installation/meterings/values/dsm-dsuid1-power
/// {
///   "data": {
///     "id":"dsm-dsuid1-power","type":"meteringValue","lastChanged":"...", "attributes":{"value":11.2}
///   }
/// }
/// ```
///
/// ### Metering values event
///
/// Metering values event is sent when some client is subscribed to `meteringValues` event type.
///
/// ```json
/// event:{
///   "type":"meteringValues",
///   "data":{
///     "values":[
///       {"id":"apartment-power","type":"meteringValue","lastChanged":"...", "attributes":{"value":11.2}},
///       {"id":"apartment-energy","type":"meteringValue","lastChanged":"...", "attributes":{"value":17.8}},
///       {"id":"dsm-dsuid1-power","type":"meteringValue","lastChanged":"...", "attributes":{"value":11.2}},
///       {"id":"dsm-dsuid1-energy","type":"meteringValue","lastChanged":"...", "attributes":{"value":17.8}}
///     ]
///   }
/// }
/// ```

#include "meterings-fwd.h"
#include <ds/enum.h>
#include <ds/clock.h>

#include "installation-fwd.h"
#include "common.h"
#include "model-fwd.h"

#include <ds/json.h>


/// Example: energyMetering
DS_REFLECTED_ENUM((dsModel), MeteringType,
    (none)
    (energyMetering)
    (powerMetering)
    (energyProducedMetering)
    (powerProducedMetering)
    (energyConsumedTariff1Metering)
    (energyConsumedTariff2Metering)
    (energyProducedTariff1Metering)
    (energyProducedTariff2Metering)
);

namespace dsModel {
struct ApartmentMeteringOrigin {
  std::string type = "apartment";

  ~ApartmentMeteringOrigin();
};

struct ControllerMeteringOrigin {
  std::string type = "controller";
  ControllerId id{};

  ~ControllerMeteringOrigin();
};

struct OpenMeteringSystemOrigin {
  std::string type = "openMeteringSystem";
  std::string serialNumber{};

  /// Example: vendor and model name
  std::string technicalName{};

  ~OpenMeteringSystemOrigin();
};

using MeteringOrigin = std::variant<std::monostate, ApartmentMeteringOrigin, ControllerMeteringOrigin, OpenMeteringSystemOrigin>;

struct MeteringAttributes {
  std::string unit{};
  std::string technicalName{};
  MeteringOrigin origin{};

  ~MeteringAttributes();
};

struct Metering {
  MeteringId id{};
  MeteringType type{};
  ds::DateTime lastChanged{};
  MeteringAttributes attributes{};

  ~Metering();
};

struct Meterings {
  std::vector<Metering> meterings{};

  ~Meterings();
};

struct MeteringValueAttributes {
  std::optional<double> value{};

  ~MeteringValueAttributes();
};

struct MeteringValue {
  MeteringId id{};
  std::string type = "meteringValue";
  ds::DateTime lastChanged{};
  MeteringValueAttributes attributes{};

  ~MeteringValue();
};

struct MeteringValues {
  std::vector<MeteringValue> values{};

  ~MeteringValues();
};

struct SeriesCompatValueValue {
  ds::DateTime timestamp{};
  std::optional<double> value{};

  ~SeriesCompatValueValue();
};


//TODO make P1 "beta" and throw this shit away
struct SeriesRealCompatValueValue {
  std::vector<double> value{};

  ~SeriesRealCompatValueValue();
};

struct SeriesCompatValue {
  std::string type = "power";

  /// Example: strip consumption/production from meteringId above
  std::string meterID{};

  /// Example: Ws
  std::string unit{};

  /// Example: 60s
  std::string resolution{};
  std::vector<SeriesRealCompatValueValue> values{};

  bool ok = true;

  ~SeriesCompatValue();
};

struct SeriesCompatValues {
  std::vector<SeriesCompatValue> values{};

  ~SeriesCompatValues();
};

} // namespace dsModel

DS_JSON_DECLARE_STRUCT((dsModel), ApartmentMeteringOrigin,
    (type)
);
DS_JSON_DECLARE_STRUCT((dsModel), ControllerMeteringOrigin,
    (type)
    (id)
);
DS_JSON_DECLARE_STRUCT((dsModel), OpenMeteringSystemOrigin,
    (type)
    (serialNumber)
    (technicalName)
);
template <>
struct DSJsonVariantTrait<dsModel::MeteringOrigin> {
  static constexpr auto discriminator = "type";
  static constexpr auto nameVisitor = ::ds::Visitor{
      [](const std::monostate &) { return ""; },
      [](const dsModel::ApartmentMeteringOrigin &) { return "apartment"; },
      [](const dsModel::ControllerMeteringOrigin &) { return "controller"; },
      [](const dsModel::OpenMeteringSystemOrigin &) { return "openMeteringSystem"; },
  };
};
DS_JSON_DECLARE((dsModel), MeteringOrigin);

DS_JSON_DECLARE_STRUCT((dsModel), MeteringAttributes,
    (unit)
    (technicalName)
    (origin)
);
DS_JSON_DECLARE_STRUCT((dsModel), Metering,
    (id)
    (type)
    (lastChanged)
    (attributes)
);
DS_JSON_DECLARE_STRUCT((dsModel), Meterings,
    (meterings)
);
DS_JSON_DECLARE_STRUCT((dsModel), MeteringValueAttributes,
    (value)
);
DS_JSON_DECLARE_STRUCT((dsModel), MeteringValue,
    (id)
    (type)
    (lastChanged)
    (attributes)
);
DS_JSON_DECLARE_STRUCT((dsModel), MeteringValues,
    (values)
);
DS_JSON_DECLARE_STRUCT((dsModel), SeriesRealCompatValueValue,
    (value)
);
DS_JSON_DECLARE_STRUCT((dsModel), SeriesCompatValueValue,
    (timestamp)
    (value)
);
DS_JSON_DECLARE_STRUCT((dsModel), SeriesCompatValue,
    (type)
    (meterID)
    (unit)
    (resolution)
    (values)
    (ok)
);
DS_JSON_DECLARE_STRUCT((dsModel), SeriesCompatValues,
    (values)
);

