/*
    Copyright (c) 2018  digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/
#pragma once

#include <trompeloeil.hpp>

#include "message-stream.h"

namespace dsNet {

struct MessageStreamMock : ds::asio::Resource<>, trompeloeil::mock_interface<MessageStream> {
  using ds::asio::Resource<>::Resource;
  IMPLEMENT_MOCK1(asyncRun);
  IMPLEMENT_MOCK2(asyncWrite);
  IMPLEMENT_MOCK1(asyncRead);
  IMPLEMENT_MOCK0(close);
  IMPLEMENT_MOCK0(sharedSession);

  std::unique_ptr<trompeloeil::expectation> allowClose{};
};

class MessageStreamClientMock : public trompeloeil::mock_interface<MessageStreamClient> {
  IMPLEMENT_MOCK0(asyncConnect);
  IMPLEMENT_CONST_MOCK0(url);
  IMPLEMENT_CONST_MOCK0(defaultUrl);
  IMPLEMENT_MOCK1(setUrl);
};

struct MessageStreamHooksMock {
  MAKE_MOCK0(opened, void());
  MAKE_MOCK1(closed, void(error_code));
  auto hooks() {
    return MessageStreamHooks{.opened = [this] { opened(); }, .closed = [this](auto&& x) { closed(DS_FWD(x)); }};
  }
};

struct MessageStreamAcceptorMock : public trompeloeil::mock_interface<MessageStreamAcceptor> {
  IMPLEMENT_MOCK1(asyncRun);
};

struct MessageStreamAcceptorHooksMock {
  MAKE_MOCK1(accept, void(ClosePtr<MessageStream>));
  auto hooks() {
    return MessageStreamAcceptorHooks{.accept = [this](auto&& x) { accept(DS_FWD(x)); }};
  }
};

} // namespace dsNet
