/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/reflect.h>
#include <gsl.h>

#include <boost/asio/io_service.hpp>
#include <boost/system/error_code.hpp>
#include <functional>
#include <memory>
#include <string>

#include "session.h"

namespace dsNet {
using boost::system::error_code;

struct MessageStreamHooks {
  std::function<void()> opened = [] {};
  std::function<void(error_code)> closed = [](auto &&) {};
};

/// Bidirection stream of framed messages.
/// New stream is created for every new connection.
struct MessageStream : virtual ISession {
public:
  virtual void asyncRun(MessageStreamHooks &&) = 0;
  using WriteHandler = std::function<void(error_code)>;
  using ReadHandler = std::function<void(error_code, std::string &&)>;
  virtual void asyncWrite(std::string &&, WriteHandler &&) = 0;
  virtual void asyncRead(ReadHandler &&) = 0;
};

/// Stream client connecting streams to server.
///
/// TODO(someday): move url to asyncConnect argument?
class MessageStreamClient {
public:
  virtual ~MessageStreamClient() = default;
  virtual ClosePtr<MessageStream> asyncConnect() = 0;
  virtual std::string url() const = 0;
  virtual std::string defaultUrl() const = 0;
  virtual void setUrl(std::string &&) = 0;
};

struct MessageStreamAcceptorHooks {
  /// Message stream was accepted and needs to be processed.
  ///
  /// The passed stream is closed when Message WSAcceptor is destroed
  /// and its stream operations will start failing.
  std::function<void(ClosePtr<dsNet::MessageStream>)> accept;
};

struct MessageStreamAcceptor : boost::noncopyable {
  virtual ~MessageStreamAcceptor() = default;
  virtual void asyncRun(MessageStreamAcceptorHooks &&) = 0;
};

} // namespace dsNet
