/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/asio/back-off.h>
#include <ds/asio/resource.h>
#include <ds/asio/timer.h>

namespace ds::asio {

using namespace std::literals;

struct RetryerPerformHooks {
  /// (synchronous) attempt to perform some operation
  std::function<void()> attempt{};
  /// attempt succeeded, attempt is not going to be called again
  std::function<void()> succeeded{};
  /// attempt failed, attempt will be called again after backoff
  std::function<void()> attemptFailed{};
};

/// Retryer performs attempts until they succeed with backoff
class Retryer : public ds::asio::Resource<> {
public:
  using ds::asio::Resource<>::Resource;

  /// Start or restart perform attempts
  void asyncPerform(RetryerPerformHooks&&);

  BackOff& backOff() { return m_backOff; }

private:
  ds::asio::BackOff m_backOff{*this, "backOff"};
  ds::asio::Timer m_timer{io()};

  void asyncRetryLoop(RetryerPerformHooks&&);
};

} // namespace ds::asio
