// Copyright (c) 2017 digitalSTROM AG, Zurich, Switzerland
//
// This file is part of digitalSTROM Server.
// vdSM is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, version 2 of the License.
//
// vdSM is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

#pragma once

// although it may drag some unneeded headers, no significant compilation time difference measured
#define CATCH_CONFIG_ENABLE_ALL_STRINGMAKERS 1

#include <catch.hpp>
#include <catch2/trompeloeil.hpp>

/// require expression to throw and exception containing given message
#define REQUIRE_THROWS_FIND(expression, message) REQUIRE_THROWS_WITH(expression, ::Catch::Matchers::Contains(message))
/// check expression to throw and exception containing given message
#define CHECK_THROWS_FIND(expression, message) CHECK_THROWS_WITH(expression, ::Catch::Matchers::Contains(message))

// namesapce ds::catch --> fix ds::Catch::use
namespace ds {

namespace catch_ {
template <typename T>
inline auto any_of(std::initializer_list<T> elements) {
  return trompeloeil::make_matcher<T>(

      // predicate lambda that checks the condition
      [](T value, std::vector<T> const& alternatives) {
        return std::any_of(std::begin(alternatives), std::end(alternatives),
                           [&value](auto element) { return value == element; });
      },

      // print lambda for error message
      [](std::ostream& os, std::vector<T> const& alternatives) {
        os << " matching any_of({";
        char const* prefix = " ";
        for (auto& element : alternatives) {
          os << prefix << element;
          prefix = ", ";
        }
        os << " }";
      },

      // stored value
      std::vector<T>(elements));
}
} // namespace catch_

/// Use the argument to avoid dead code elimination
// from https://stackoverflow.com/questions/40122141/preventing-compiler-optimizations-while-benchmarking
template <typename T>
static void use(T&& t) {
  __asm__ __volatile__("" ::"g"(t));
}

} // namespace ds
