#include "test.h"
#include <ds/log.h>
#include <boost/range/algorithm/find.hpp>
#include "messages.h"

static ds::LogChannel dsLogChannel("dsLocaleTest");

namespace ds {
namespace locale {
namespace test {

class MessagesImpl : public Messages {
public:
  MessagesImpl() { m_domains.push_back(""); }

  const char *get(int domainId, const char *context, const char *msgId) const override {
    DS_REQUIRE(unsigned(domainId) < m_domains.size());
    auto &&domain = m_domains.at(domainId);
    // Empty domain is default domain in test locale. Strip it.
    auto &&domainStr = domain.empty() ? std::string() : ds::str(domain, "|");
    if (context) {
      m_strings.push_back(std::make_unique<std::string>(ds::str(domainStr, context, "|", msgId)));
    } else {
      m_strings.push_back(std::make_unique<std::string>(ds::str(domainStr, msgId)));
    }
    return m_strings.back()->c_str(); // cache until end of live to avoid dangling pointers
  }

  const char *get(int domainId, const char *context, const char *msgId, int n) const override {
    DS_REQUIRE(unsigned(domainId) < m_domains.size());
    auto &&domain = m_domains.at(domainId);
    // Empty domain is default domain in test locale. Strip it.
    auto &&domainStr = domain.empty() ? std::string() : ds::str(domain, "|");
    if (context) {
      m_strings.push_back(std::make_unique<std::string>(ds::str(domainStr, context, "|", msgId, "|", n)));
    } else {
      m_strings.push_back(std::make_unique<std::string>(ds::str(domainStr, msgId, "|", n)));
    }
    return m_strings.back()->c_str(); // cache until end of live to avoid dangling pointers
  }

  int domain(const std::string &name) const override {
    auto domainIt = boost::find(m_domains, name);
    if (domainIt != m_domains.end()) {
      return domainIt - m_domains.begin();
    }
    return addDomain(name);
  }

  const char *convert(const char *msg, string_type &buffer) const override { return msg; }

  const std::string &language() const override {
    static std::string x("test");
    return x;
  }

protected:
  // std::locale is the only one who can own and delete facets
  ~MessagesImpl() {}

private:
  mutable std::vector<std::string> m_domains;
  mutable std::vector<std::unique_ptr<std::string>> m_strings;

  int addDomain(const std::string &name) const {
    m_domains.push_back(name);
    auto &&id = m_domains.size() - 1;
    DS_DEBUG_LEAVE(name, id);
    return id;
  }
};

std::locale makeLocale() { return std::locale(std::locale::classic(), new MessagesImpl()); }

} // namespace test
} // namespace locale
} // namespace ds
