/*
    Copyright (c) 2022 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <digitalSTROM/dsuid.h>

#include <optional>

#include "metering/config.h"
#include "metering/metering.h" // ArchiveValue

namespace dss::metering {

class Direction {
public:
  virtual ~Direction() = default;

  virtual ValueRange getSeries(std::vector<Dsuid> meterIds, std::chrono::seconds resolution, SeriesTypes _type,
                               bool _energyInWh, const TimeDuration& timeDuration, int _valueCount) const = 0;
  virtual double get15MinuteAverage(std::vector<Dsuid> meterIds, metering::SeriesTypes type, Unit unit) const = 0;
  virtual double getLastEnergy(const dsuid_t& meterId) const = 0;
  virtual std::optional<ArchiveValue> getEnergy(const dsuid_t& meterId) const = 0;
  virtual std::vector<dsuid_t> getActiveMeters() const = 0;

  virtual double getInstantPower(const dsuid_t& meterId) const = 0;
  virtual unsigned int getApartmentPower() const = 0;

  virtual void addEnergy(const dsuid_t& meterId, ElectricityMeasurement&& energyMeasurement) = 0;
  virtual void pollTimestamp(ds::DateTime timepoint) = 0;
};

std::unique_ptr<Direction> makeDirection(Archives* archives, Category category, ElectricityMeteringConfig&);
} // namespace dss::metering
