/*
    Copyright (c) 2019 digitalSTROM AG, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef __DSS_MODEL_FLOOR_H__
#define __DSS_MODEL_FLOOR_H__

#include <ds-model/groups-fwd.h>
#include <ds-model/installation-fwd.h>

#include <memory>
#include <string>
#include <vector>

namespace dss {

class Apartment;
class PropertyNode;
typedef std::shared_ptr<PropertyNode> PropertyNodePtr;

class Floor
{
public:
  Floor(int id, Apartment *apartment);
  int getID() const;
  dsModel::FloorId id() const;
  void setOrder(int position);
  int getOrder() const;
  const std::string& getName() const;
  void setName(std::string name);
  std::vector<int> getZones() const { return m_zones; }
  std::vector<dsModel::ZoneId> zones() const;
  void addZone(int id);
  void removeZone(int id);
  void publishToPropertyTree();
  void removeFromPropertyTree();

  bool isInDsModel() const { return true; }

private:
  int m_id;
  int m_order;
  std::string m_name;
  std::vector<int> m_zones;
  Apartment *m_apartment;
  PropertyNodePtr m_propertyNode;
};

}

#endif//__DSS_MODEL_FLOOR_H__
