#pragma once

#include <boost/asio.hpp>
#include <chrono>
#include <string>

#include "config-file.h"
#include "parser.h"
#include "socket-writer.h"

namespace dss::p1 {

struct SenderQueue {
  SenderQueue(boost::asio::io_context& io, std::string socketPath) : m_io(io), m_socketPath(std::move(socketPath)) {}

  void send(MeterReading&& meterReading);

  boost::asio::io_context& m_io;
  std::string m_socketPath;
};

struct FrameBuffer {
  // FrameBuffer(int capacity) : m_data{boost::circular_buffer<char>(capacity)} {}
  FrameBuffer(SenderQueue& senderQueue, size_t capacity, MeterParser& parser)
      : m_senderQueue(senderQueue), m_frame(std::vector<char>(capacity)), m_parser(parser) {
    m_frame.clear(); // m_frame.size() == capacity
  }

  void processFrame();

  using data_t = std::vector<char>;
  void push(data_t::const_iterator begin, data_t::const_iterator end);

  int getCrcErrorCount() { return m_parser.getCrcErrorCount(); }

  int getOverflowCount() { return m_overflowCount; }

  void clearErrors() {
    m_parser.resetCrcCount();
    m_overflowCount = 0;
  }

  // boost::circular_buffer<char> m_data;
  SenderQueue& m_senderQueue;
  std::vector<char> m_frame;
  MeterParser& m_parser;
  int m_sum{};
  int m_overflowCount{};
};

struct SerialConnection {
  SerialConnection(boost::asio::io_context& io, SerialPortConfig&& config, FrameBuffer& frameBuffer)
      : m_io(io),
        m_config(config),
        m_readBuffer(std::vector<char>(config.readBufferCapacity)),
        m_frameBuffer(frameBuffer) {
    m_lastDataReceived = std::chrono::system_clock::now();
  }

  void start();

  void doRead();

  std::vector<std::string> getSerialPorts();

  std::optional<std::string> getNextPort();

  bool isDataStale();

  std::string m_activePortPath = "";

  boost::asio::io_context& m_io;
  SerialPortConfig m_config;
  std::vector<char> m_readBuffer{};
  std::shared_ptr<boost::asio::serial_port> m_serialPort{};

  bool m_isReading{false};
  FrameBuffer& m_frameBuffer;
  std::chrono::time_point<std::chrono::system_clock> m_lastDataReceived;
};

} // namespace dss::p1