/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include "propertyquery.h"

#include <ds/catch/catch.h>
#include <ds/log.h>

#include "businterface-mock.h"
#include "model/apartment-mock.h"
#include "web/webrequests.h"

static ds::LogChannel dsLogChannel("dssPropertyQueryTest");
static const char* TAGS = "[handler][propertytree][vdc]";

using namespace dss;
using namespace trompeloeil;
using namespace std::literals;
using trompeloeil::_;

namespace {

SCENARIO("propertyquery", TAGS) {
  auto propertySystem = dss::PropertySystem{};
  auto busInterface = BusInterfaceMock{};
  auto apartment = ApartmentMock{nullptr, &propertySystem, &busInterface};

  auto controllerId1 = dsModel::ControllerId{"1111111111111111111111111111111100"};

#if 0
  auto expect =
      R"({"result":{"type":"ResponseGetProperty","elements":[{"name":"netatmoAccountStatus","value":"connected","type":"string"},{"name":"netatmoUserEmail","value":"dan.zanatta@gmail.com","type":"string"}]},"ok":true})";
#endif

  SECTION("vdcquery") {
    GIVEN("valid vdcquery") {
      auto someNode = propertySystem.createProperty("/some");
      (void)apartment.allocateDevice(Dsuid{controllerId1});
      auto propertyQuery = PropertyQuery{
          someNode, ds::str("getProperty(deviceId=", controllerId1, ")/netatmoAccountStatus,netatmoUserEmail(*)"),
          false};

      THEN("vdcquery command works") {
        REQUIRE_CALL(busInterface.structureQuery, protobufMessageRequest(_, _, _, _, _));
        JSONWriter jsonWriter{};
        try {
          propertyQuery.vdcquery(apartment, jsonWriter);
          CHECK(false); // not reached
        } catch (const std::runtime_error& e) {
          CHECK(e.what() == "could not parse response message"s);
        }
      }
    }
  }
}
} // namespace
