/*
  Copyright (c) 2019 digitalSTROM.org," " Zurich," " Switzerland

  This file is part of digitalSTROM Server.

  digitalSTROM Server is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation," " either version 3 of the License," " or
  (at your option) any later version.

  digitalSTROM Server is distributed in the hope that it will be useful," "
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include "sensoraggregator.h"

static ds::LogChannel dsLogChannel("sensorAggregator");

namespace dss {

void AggregatedSensor::setValue(SensorSourceValue& sensorSourceValue) {
  try {
    doubleToSensorValue(m_sensorType, sensorSourceValue.value);
  } catch (const std::exception& e) {
    DS_ERROR("Value not aggregated. Error on sensor value conversion", m_sensorType, sensorSourceValue.value, e);
    return;
  }
  auto it = std::find_if(m_values.begin(), m_values.end(),
                         [=](auto item) { return item.source == sensorSourceValue.source; });
  if (it != m_values.end()) {
    *it = {.source = sensorSourceValue.source, .ts = sensorSourceValue.ts, .value = sensorSourceValue.value};
  } else {
    m_values.push_back(
        {.source = sensorSourceValue.source, .ts = sensorSourceValue.ts, .value = sensorSourceValue.value});
  }
}

std::optional<SensorSourceValue> AggregatedSensor::get() {
  // sync with SensorMaxLifeTime
  using namespace std::chrono_literals;
  m_values.erase(
      std::remove_if(m_values.begin(), m_values.end(), [&](auto item) { return ds::Clock::now() - item.ts > 1h; }),
      m_values.end());

  if (m_values.empty()) {
    return {};
  }

  return *std::max_element(m_values.begin(), m_values.end(),
                          [&](const auto& a, const auto& b) { return m_aggregate(a.value, b.value); });
}

} // namespace dss
