/*
  Copyright (c) 2019 digitalSTROM.org," " Zurich," " Switzerland

  This file is part of digitalSTROM Server.

  digitalSTROM Server is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation," " either version 3 of the License," " or
  (at your option) any later version.

  digitalSTROM Server is distributed in the hope that it will be useful," "
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include "model/device.h"

namespace dss {
struct SensorSourceValue {
  dsuid_t source;
  ds::Clock::time_point ts;
  double value;
};

class AggregatedSensor {
  std::vector<SensorSourceValue> m_values;
  std::function<double(double, double)> m_aggregate;
  SensorType m_sensorType;

public:
  AggregatedSensor(std::function<double(double, double)>&& aggregate, SensorType _sensorType)
      : m_aggregate(std::move(aggregate)), m_sensorType(_sensorType) {}

  void setValue(SensorSourceValue& sensorSourceValue);
  std::optional<SensorSourceValue> get();

  static constexpr auto max = [](double a, double b) { return a < b; };
  static constexpr auto min = [](double a, double b) { return a > b; };
};

class AggregatedWindSpeed : public AggregatedSensor {
public:
  AggregatedWindSpeed() : AggregatedSensor(max, SensorType::WindSpeed) {}
};

class AggregatedWindGust : public AggregatedSensor {
public:
  AggregatedWindGust() : AggregatedSensor(max, SensorType::GustSpeed) {}
};

class AggregatedTemperature : public AggregatedSensor {
public:
  AggregatedTemperature() : AggregatedSensor(min, SensorType::TemperatureOutdoors) {}
};

class AggregatedBrightness : public AggregatedSensor {
public:
  AggregatedBrightness() : AggregatedSensor(max, SensorType::BrightnessOutdoors) {}
};

class AggregatedHumidity : public AggregatedSensor {
public:
  AggregatedHumidity() : AggregatedSensor(max, SensorType::HumidityOutdoors) {}
};

class AggregatedPrecipitation : public AggregatedSensor {
public:
  AggregatedPrecipitation() : AggregatedSensor(max, SensorType::Precipitation) {}
};

class AggregatedAirPressure : public AggregatedSensor {
public:
  AggregatedAirPressure() : AggregatedSensor(min, SensorType::AirPressure) {}
};

} // namespace dss
