/*
    Copyright (c) 2009 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <memory>

#include "businterface.h"
#include "model/behavior.h"
#include "model/apartment.h"
#include "model/devicereference.h"

namespace dss {

class Apartment;
class DSMeter;
class Device;
class Zone;
class Group;
class Cluster;
class Set;
struct TemperatureConfig;

class StructureManipulator {
private:
  Apartment& m_Apartment;
  StructureModifyingBusInterface& m_Interface;
  class StructureQueryBusInterface& m_QueryInterface;
  void checkSensorsOnDeviceRemoval(Zone& _zone, Device& _device);
  void wipeZoneOnMeter(dsuid_t _meterDSUID, int _zoneID);
  void autoAssignZoneSensorsImpl(Zone& _zone);

public:
  StructureManipulator(Apartment& apartment);

  void createZone(std::shared_ptr<DSMeter> _dsMeter, std::shared_ptr<Zone> _zone);
  void addDeviceToZone(std::shared_ptr<Device> _device, std::shared_ptr<Zone> _zone);
  void removeZoneOnDSMeter(std::shared_ptr<Zone> _zone, std::shared_ptr<DSMeter> _dsMeter);
  void removeZoneOnDSMeters(std::shared_ptr<Zone>);
  void removeDeviceFromDSMeter(std::shared_ptr<Device> _device);
  /**
   * @ret list of devices (slaves included) that have been deleted
   */
  std::vector<std::shared_ptr<Device> > removeDevice(std::shared_ptr<Device> _pDevice);
  void deviceSetName(std::shared_ptr<Device> _pDevice, const std::string& _name);
  void meterSetName(std::shared_ptr<DSMeter> _pMeter, const std::string& _name);
  int persistSet(Set& _set, const std::string& _originalSet);
  int persistSet(Set& _set, const std::string& _originalSet, int _groupNumber);
  void unpersistSet(std::string _setDescription);

  void createGroup(std::shared_ptr<Zone> _zone, int _groupNumber, ApplicationType applicationType,
                   const int applicationConfiguration, const std::string& _name);
  void removeGroup(std::shared_ptr<Zone> _zone, int _groupNumber);
  void removeClusterStates(std::shared_ptr<Cluster> _cluster);
  void groupSetName(std::shared_ptr<Group> _group, const std::string& _name);
  void groupSetApplication(std::shared_ptr<Group> _group, ApplicationType applicationType,
                           const int applicationConfiguration);

  void sceneSetName(std::shared_ptr<Group> _group, int _sceneNumber, const std::string& _name);
  void saveScene(IDeviceInterface&, CallOrigin, int sceneId, const std::string& token);

  void clusterSetName(std::shared_ptr<Cluster> _cluster, const std::string& _name);
  void clusterSetApplication(std::shared_ptr<Cluster> _cluster, ApplicationType applicationType,
                             const int applicationConfiguration);
  void clusterSetConfigurationLock(std::shared_ptr<Cluster> _cluster, bool _locked);

  void deviceAddToGroup(std::shared_ptr<Device> _device, std::shared_ptr<Group> _group);
  void deviceRemoveFromGroup(std::shared_ptr<Device> _device, std::shared_ptr<Group> _group,
                             bool _adaptInputs = true);
  void deviceRemoveFromGroups(std::shared_ptr<Device> device);
  void deviceRemoveFromClusters(std::shared_ptr<Device> device);

  void setDeviceZoneHeatingConfig(const std::shared_ptr<const DSMeter>& meter, Zone& zone);
  void setZoneHeatingConfig(std::shared_ptr<Zone> zone, const TemperatureConfig& spec);
  void setZoneHeatingConfigSpec(std::shared_ptr<Zone> zone, const ZoneHeatingConfigSpec_t& spec);
  void setNextTemperatureController(Zone&);
  void electTemperatureController(Zone&);
  void electTemperatureControllers();
  void setZoneSensor(Zone& _zone, SensorType _sensorType, std::shared_ptr<Device> _dev);
  void resetZoneSensor(Zone& _zone, SensorType _sensorType);
  void autoAssignZoneSensors(Zone& _zone);
  void synchronizeZoneSensorAssignment(Zone& zone);
  /**
   * @ret device moved
   */
  bool setJokerGroup(std::shared_ptr<Device> device, int groupId);

  void setProperty(std::shared_ptr<DSMeter> _dsMeter,
                   const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& properties);
  vdcapi::Message getProperty(std::shared_ptr<DSMeter> _dsMeter,
                              const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& query);
}; // StructureManipulator

} // namespace dss
