/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "controller-statuses.h"

#include <ds/log.h>

static ds::LogChannel dsLogChannel("testDevicePresence");

namespace dss {

struct BackendVdcTestPresenceResult {
  bool active;
};

struct DsaTestPresenceResult {
  bool present;
};

bool testDevicePresence(const std::locale& locale, const Device& device, DsaHttpClient& dsa, BackendVdcs& backendVdcs) {
  if (backendVdcs.isBackendVdcDevice(device.getDSID())) {
    return ds::fromJson<BackendVdcTestPresenceResult>(
               backendVdcs.http(
                   HttpRequest{.url = ds::str("/v1/cloudvdc/vdcs/", backendVdcs.vdcIdForDeviceDsuid(device.getDSID()),
                                              "/devices/", device.getDSID(), "/action/testPresence"),
                               .type = HttpMethod::POST

                   },
                   locale))
        .active;
  } else {
    return ds::fromJson<DsaTestPresenceResult>(
               dsa.request(dss::HttpRequest{.url = ds::str("/dsa/json/device/testDevicePresence?dsuid=",
                                                           device.getDSID(), "&dsm_dsuid=", device.getDSMeterDSID())}))
        .present;
  }
}
} // namespace dss

DS_REFLECT_STRUCT((dss), BackendVdcTestPresenceResult, (active));
DS_REFLECT_STRUCT((dss), DsaTestPresenceResult, (present));
