/*
 *  Copyright (c) 2014 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "dss.h"
#include "model/group.h"
#include "model/modelconst.h"
#include "vdc-messages.pb.h"

namespace dss {

class DummyStructureModifyingInterface : public StructureModifyingBusInterface {
public:
  std::string name() const override { return "dummy"; }
  void setZoneID(const dsuid_t& _dsMeterID, const devid_t _deviceID, const int _zoneID) override {}
  void createZone(const dsuid_t& _dsMeterID, const int _zoneID) override {}
  void removeZone(const dsuid_t& _dsMeterID, const int _zoneID) override {}
  void addToGroup(const dsuid_t& _dsMeterID, const int _groupID, const int _deviceID) override {}
  void removeFromGroup(const dsuid_t& _dsMeterID, const int _groupID, const int _deviceID) override {}
  void removeDeviceFromDSMeter(const dsuid_t& _dsMeterID, const int _deviceID) override {}
  void removeDeviceFromDSMeters(const dsuid_t& _deviceDSID) override {}
  void sceneSetName(uint16_t _zoneID, uint8_t _groupID, uint8_t _sceneNumber, const std::string& _name) override {}
  void deviceSetName(dsuid_t _meterDSID, devid_t _deviceID, const std::string& _name) override {}
  void meterSetName(dsuid_t _meterDSID, const std::string& _name) override {}
  void createGroup(uint16_t _zoneID, uint8_t _groupID, ApplicationType applicationType, uint32_t applicationConfig,
                   const std::string& _name) override {}
  void removeGroup(uint16_t _zoneID, uint8_t _groupID) override {}
  void groupSetName(uint16_t _zoneID, uint8_t _groupID, const std::string& _name) override {}
  void createCluster(uint8_t _groupID, ApplicationType applicationType, uint32_t applicationConfig,
                     const std::string& _name) override {}
  void removeCluster(uint8_t _clusterID) override {}
  void groupSetApplication(uint16_t _zoneID, uint8_t _groupID, ApplicationType applicationType,
                           uint32_t applicationConfig) override {}
  void clusterSetName(uint8_t _clusterID, const std::string& _name) override {}
  void clusterSetApplication(uint8_t _clusterID, ApplicationType applicationType, uint32_t applicationConfig) override {
  }
  void clusterSetProperties(uint8_t _clusterID, uint16_t _location, uint16_t _floor,
                            uint16_t _protectionClass) override {}
  void clusterSetLockedScenes(uint8_t _clusterID, const std::vector<int>& lockedScenes) override {}
  void clusterSetConfigurationLock(uint8_t _clusterID, bool _lock) override {}
  void clusterSetOperationLock(uint8_t _clusterID, bool _lock) override {}
  void setButtonSetsLocalPriority(const dsuid_t& _dsMeterID, const devid_t _deviceID, bool _setsPriority) override {}
  void setButtonCallsPresent(const dsuid_t& _dsMeterID, const devid_t _deviceID, bool _callsPresent) override {}
  void setZoneHeatingConfig(const dsuid_t& _dsMeterID, const uint16_t _ZoneID,
                            const ZoneHeatingConfigSpec_t _spec) override {}
  void setZoneHeatingOperationModes(const dsuid_t& _dsMeterID, const uint16_t _ZoneID,
                                    const ZoneHeatingOperationModeSpec_t _spec) override {}
  void setZoneSensor(const uint16_t _zoneID, SensorType _sensorType, const dsuid_t& _sensorDSUID) override {}
  void resetZoneSensor(const uint16_t _zoneID, SensorType _sensorType) override {}
  void setCircuitPowerStateConfig(const dsuid_t& _dsMeterID, const int _index, const int _setThreshold,
                                  const int _resetThreshold) override {}
  void setMeteringConfiguration(const dsuid_t& dsm, const MeteringConfigurationSpec_t& spec) override {}
  void setProperty(const dsuid_t& _meter,
                   const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& properties) override {}
  vdcapi::Message getProperty(const dsuid_t& _meter,
                              const ::google::protobuf::RepeatedPtrField< ::vdcapi::PropertyElement>& query) override {
    return vdcapi::Message();
  }

  void setFireMuteEnabled(dsuid_t dsm, bool) override {}
  void removeVdc(const dsuid_t& vdsmID, const dsuid_t& vdcID) override {}

  void clusterSetStopTimeout(uint8_t, uint8_t) override {}
  void zoneSetStopTimeout(uint16_t, uint8_t, uint8_t) override {}
};

class DummyStructureQueryBusInterface : public StructureQueryBusInterface {
public:
  virtual std::vector<DSMeterSpec_t> getBusMembers() { return std::vector<DSMeterSpec_t>(); }
  virtual DSMeterSpec_t getDSMeterSpec(const dsuid_t& _dsMeterID) { return DSMeterSpec_t(); }
  virtual std::vector<int> getZones(const dsuid_t& _dsMeterID) { return std::vector<int>(); }
  virtual std::vector<DeviceSpec> getDevicesInZone(const dsuid_t& _dsMeterID, const int _zoneID, bool complete = true) {
    return std::vector<DeviceSpec>();
  }
  virtual std::vector<DeviceSpec> getInactiveDevicesInZone(const dsuid_t& _dsMeterID, const int _zoneID) {
    return std::vector<DeviceSpec>();
  }
  virtual std::vector<GroupSpec_t> getGroups(const dsuid_t& _dsMeterID, const int _zoneID) {
    return std::vector<GroupSpec_t>();
  }
  virtual std::vector<ClusterSpec_t> getClusters(const dsuid_t& _dsMeterID) { return std::vector<ClusterSpec_t>(); }
  virtual std::vector<CircuitPowerStateSpec_t> getPowerStates(const dsuid_t& _dsMeterID) {
    return std::vector<CircuitPowerStateSpec_t>();
  }
  virtual MeteringConfigurationSpec_t getMeteringConfiguration(const dsuid_t& dsm) {
    return MeteringConfigurationSpec_t();
  }
  virtual std::vector<std::pair<int, int> > getLastCalledScenes(const dsuid_t& _dsMeterID, const int _zoneID) {
    return std::vector<std::pair<int, int> >();
  }
  virtual std::bitset<7> getZoneStates(const dsuid_t& _dsMeterID, const int _zoneID) { return std::bitset<7>(0llu); }

  virtual bool getEnergyBorder(const dsuid_t& _dsMeterID, int& _lower, int& _upper) {
    _lower = 0;
    _upper = 0;
    return false;
  }
  virtual DeviceSpec deviceGetSpec(devid_t _id, dsuid_t _dsMeterID) { return DeviceSpec(); }
  virtual std::string getSceneName(dsuid_t _dsMeterID, std::shared_ptr<Group> _group, const uint8_t _sceneNumber) {
    return "";
  }
  virtual DSMeterHash_t getDSMeterHash(const dsuid_t& _dsMeterID) { return DSMeterHash_t(); }
  virtual void getDSMeterState(const dsuid_t& _dsMeterID, uint8_t* _state) { *_state = 0; }
  virtual ZoneHeatingConfigSpec_t getZoneHeatingConfig(const dsuid_t& _dsMeterID, const uint16_t _ZoneID) {
    return ZoneHeatingConfigSpec_t();
  }
  virtual ZoneHeatingInternalsSpec_t getZoneHeatingInternals(const dsuid_t& _dsMeterID, const uint16_t _ZoneID) {
    return ZoneHeatingInternalsSpec_t();
  }
  virtual ZoneHeatingStateSpec_t getZoneHeatingState(const dsuid_t& _dsMeterID, const uint16_t _ZoneID) {
    return ZoneHeatingStateSpec_t();
  }
  virtual ZoneHeatingOperationModeSpec_t getZoneHeatingOperationModes(const dsuid_t& _dsMeterID,
                                                                      const uint16_t _ZoneID) {
    return ZoneHeatingOperationModeSpec_t();
  }
  virtual dsuid_t getZoneSensor(const dsuid_t& _meterDSUID, const uint16_t _zoneID, SensorType _sensorType) {
    return DSUID_NULL;
  }
  virtual void getZoneSensorValue(const dsuid_t& _meterDSUID, const uint16_t _zoneID, SensorType _sensorType,
                                  uint16_t* _sensorValue, uint32_t* _sensorAge) {}
  virtual int getDevicesCountInZone(const dsuid_t& _dsMeterID, const int _zoneID, bool _onlyActive = false) {
    return 0;
  }
  virtual void protobufMessageRequest(const dsuid_t _dSMdSUID, const uint16_t _request_size, const uint8_t* _request,
                                      uint16_t* _response_size, uint8_t* _response) {}
}; // DummyStructureQueryBusInterface

class DummyActionRequestInterface : public ActionRequestInterface {
public:
  void callScene(AddressableModelItem* pTarget, const uint16_t scene, Caller&& caller, const bool _force) override {
    Group* pGroup = dynamic_cast<Group*>(pTarget);
    Device* pDevice = dynamic_cast<Device*>(pTarget);
    m_Log += "callScene(";
    if (pGroup != NULL) {
      m_Log += intToString(pGroup->getZoneID()) + "," + intToString(pGroup->getID());
    } else {
      m_Log += intToString(pDevice->getShortAddress());
    }
    m_Log += "," + intToString(scene) + ")";
  }
  void callSceneMin(AddressableModelItem* pTarget, const uint16_t _scene, Caller&& caller) override {}
  void saveScene(AddressableModelItem* pTarget, const uint16_t scene, Caller&& caller) override {}
  void undoScene(AddressableModelItem* pTarget, const uint16_t scene, Caller&& caller) override {}
  void undoSceneLast(AddressableModelItem* pTarget, Caller&& caller) override {}
  void blink(AddressableModelItem* pTarget, Caller&& caller) override {}
  void setValue(AddressableModelItem* pTarget, const uint8_t _value, Caller&& caller) override {}
  void increaseOutputChannelValue(AddressableModelItem*, ChannelType, Caller&&) override {}
  void decreaseOutputChannelValue(AddressableModelItem*, ChannelType, Caller&&) override {}
  void stopOutputChannelValue(AddressableModelItem*, ChannelType, Caller&&) override {}
  void pushSensor(AddressableModelItem*, dsuid_t _sourceID, SensorType _sensorType, double _sensorValueFloat,
                  Caller&& caller) override {}
  void muteFire() override {}
  void setState(Group* group, const uint8_t zoneState, const uint8_t areaState1, const uint8_t areaState2,
                const uint8_t areaState3, const uint8_t areaState4) override {}
  void setState(Cluster* cluster, const uint8_t state) override {}
  void stopStopTimer(Group* group, const uint8_t zoneOrArea) override {}
  void stopStopTimer(Cluster* cluster) override {}

  bool isOperationLock(const dsuid_t& _dSM, int _clusterId) override { return false; }

  std::string getLog() { return m_Log; }

  void clearLog() { m_Log.clear(); }

private:
  std::string m_Log;
};

struct DummyBusInterface : public BusInterface {
  DeviceBusInterface* getDeviceBusInterface() override { return NULL; }
  StructureQueryBusInterface* getStructureQueryBusInterface() override { return &dummyStructureQueryBusInterface; }
  MeteringBusInterface* getMeteringBusInterface() override { return NULL; }
  StructureModifyingBusInterface* getStructureModifyingBusInterface() override {
    return &dummyStructureModifyingInterface;
  }
  ActionRequestInterface* getActionRequestInterface() override { return &dummyActionRequestInterface; }

  void setBusEventSink(BusEventSink*) override {}
  const std::string getConnectionURI() override { return {}; }

  // allow direct access to clearLog etc
  DummyStructureModifyingInterface dummyStructureModifyingInterface{};
  DummyStructureQueryBusInterface dummyStructureQueryBusInterface{};
  DummyActionRequestInterface dummyActionRequestInterface{};
};

} // namespace dss
