// clang-format off

(function () {
    let connection = null;

    function connect () {
        try {
            connection = new signalR.HubConnectionBuilder()
                .withUrl(
                    // `wss://${window.location.hostname}/hub/v1/apartment/notifications`,
                    `wss://${window.location.hostname}/api/v1/apartment/notifications`,
                    {
                        skipNegotiation: true,
                        transport: signalR.HttpTransportType.WebSockets
                    }
                )
                .configureLogging(signalR.LogLevel.Information)
                .build();
            connection.onclose(start);
            console.log('SignalR connected.');
            
            start();
        } catch (e) {
            console.log('SignalR NOT connected.');
            console.log(e);
        }
    }
    
    async function start () {
        try {
            await connection.start();
            console.log('SignalR connection started.');
            await subscribeToMeterings();
            console.log('Subscribed to meterings.');
            
            const messages = document.getElementById('messages');
            connection.on('event', (message) => {
                if (message) {
                    const p = document.createElement('p');
                    p.textContent = `[${new Date().toISOString()}] Got notification: ${JSON.stringify(message)}`;
                    messages.append(p);
                }
            });
        } catch (err) {
            console.log('SignalR connection NOT started.');
            console.log(err);
        }
    }

    function subscribeToMeterings () {
        return connection.send('subscribe', { type: 'meteringValues', timeout: 3000 });
    }

    // Start the connection.
    const startButton = document.getElementById('start');
    startButton.addEventListener('click', connect);
})();
